
#include "shared.h"

char *lang;
unsigned short version = 0x0000;
unsigned short x_flags = 0x0000;
unsigned short pers = 0x0000;	// MinGW error if this is before version :S

void print_version() 
{
	char *plang; // print language
	char vers[16];
	sprintf (vers, "%d.%d%d%s", (version >> 12) & 0x0f, (version >> 8) & 0x0f, (version >> 4) & 0x0f, (version & 0x0f) ? " (Beta)" : "");
	if ((x_flags & 0x17) == 0x03) lang = "Japanese";
	if ((x_flags & 0x17) == 0x05) lang = "English";
	if ((x_flags & 0x17) == 0x07) lang = "Simplified Chinese";
	if ((x_flags & 0x17) == 0x17) lang = "Traditional Chinese";

	plang = lang;
	if ((x_flags & 0x77) == 0x25) plang = "French";
	if ((x_flags & 0x77) == 0x45) plang = "Korean";

	printf ("%s %s V. %s CRC(%8.8x)\n", (x_flags & 8) ? "R4DS" : "M3DSS", plang, vers, nCrc);
}

void get_user_input()
{
	char dst[16];

	char *outlang[3] = {
		"English or Chinese",
		"Chinese or Japanese",
		"English or Japanese"
	};

	printf ("\nThis file is meant for use on %s hardware, do you", ~x_flags&8 ? "M3" : "R4");
	printf ("\nwant to patch it for use on %s hardware? (Yes or No)\n",   x_flags&8 ? "M3" : "R4");
	gets (dst);

	if (strncmp (dst, "Y", 1) == 0 || strncmp (dst, "y", 1) == 0)
		flags |= 0x08;

	printf ("\nThis file is meant for use on %s hardware, do you", lang);
	printf ("\nwant to patch it for use on %s hardware?\n(%s or No)\n", outlang[((x_flags>>1)&3)-1], outlang[((x_flags>>1)&3)-1]);
	gets (dst);

	dst[0] &= ~0x20;

	int tst = 0;
	if (strncmp (dst, outlang[((x_flags>>1)&3)-1] + 0, strlen(dst)) == 0)
		tst = 1;

	if (strncmp (dst, outlang[((x_flags>>1)&3)-1] + 11, strlen(dst)) == 0)
		tst = 2;

	if (strncmp (dst, "1", 1) == 0 || strncmp (dst, "A", 1) == 0)
		tst = 1;

	if (strncmp (dst, "2", 1) == 0 || strncmp (dst, "B", 1) == 0)
		tst = 2;

	if (tst)
	{
		switch (((x_flags>>1)&3)-1)
		{
			case 0:
				flags |= (tst - 1) ? 0x04 : 0x06; // English or Chinese
			break;

			case 1:
				flags |= (tst - 1) ? 0x06 : 0x02; // Chinese or Japanese
			break;

			case 2:
				flags |= (tst - 1) ? 0x04 : 0x02; // English or Japanese
			break;
		}
	}

	if ((version >= 0x0109 && ~x_flags & 8) || (version >= 0x0105 && x_flags & 8))
	{
		printf ("\nWould you like to remove the save file size check? (Yes or No)\n");
		gets (dst);

		if (strncmp (dst, "Y", 1) == 0 || strncmp (dst, "y", 1) == 0)
			x_flags |= 0x8000;
	}
}

void search_info()
{
	int i;
	for (i = 0; i < nLen; i++)
	{
		if (memcmp (src + i, cmp, 0x0a) == 0 && memcmp (src + i + 0x0b, cmp + 0x0b, 0x03) == 0 && x_flags == 0)
		{
			x_flags = src[i + 0x0a] & 0x0f;

			if (version) break;
		}

		if (src[i] == 0x00 && src[i+1] == 0x76 && (src[i+2] == 0x31 || src[i+2] == 0x32) && src[i+3] == 0x2E && version == 0)
		{
			version = ((src[i+2] & 0x0f) << 12) | ((src[i+4] & 0x0f) << 8) | ((src[i+5] & 0x0f) << 4);
			if (src[i+6] == 0x20 && (src[i+7]&0xdf) == 0x42)
				version |= 0x0b;

			if (x_flags) break;
		}
	}

	if (flags & 8) // r4
	{
		switch (version & 0xfff0)
		{
			case 0x1080:
				if ((version & 0x0f) == 0)
					pers = 1;
			break;
	
			case 0x1090:
				pers = 2;
			break;
			case 0x1100:
				pers = 3;
			break;	
			default:
				pers = 0;
			break;
		}
	}
	else	// m3
	{
		switch (version & 0xfff0)
		{
			case 0x1040:
				if ((version & 0x0f) == 0)
					pers = 1;
			break;
	
			case 0x1050:
				pers = 2;
			break;
			case 0x1060:
				pers = 3;
			break;	
			default:
				pers = 0;
			break;
		}
	}
}	


void get_patch_addresses(FILE *fz, int nNum)
{
	int i;
	int tNum = 0;
	int nStart = 0;
	char szLine[256];
	char tmp[32];
	memset(addresses, 0, 32);

	rewind (fz);	// move pointer back to beginning

	while (1)
	{
		if (fgets(szLine, 256, fz) == NULL)
			break;

		int rLen = strlen(szLine);

		if (strncmp ("[PATCH]", szLine, 7) == 0) {
			nStart = 1;
			continue;
		}

		if (nStart == 0)
			continue;

		// Find commas ('cause I like to do it this way)
		int c0[5], c1 = 0;
		for (i = 0; i < rLen; i++) {
			if (szLine[i] == ',') {
				c0[c1] = i;
				c1++;
			}
			if (c1 == 32 || szLine[(i+1) & 0xff] == '/')
				break;
		}

		tNum = 0;
		memcpy (tmp, szLine, c0[0]);
		tmp[c0[0]] = '\0';
		sscanf (tmp, "%d", &tNum);

		if (tNum != nNum)
			continue;

		for (i = 0; i < (c1 & 0x1f) - 1; i++)
		{
			memcpy (tmp, szLine + c0[i] + 1, c0[i+1]-c0[i]);
			tmp[c0[i+1]-c0[i]] = '\0';
			sscanf (tmp, "%hX", &addresses[i]);
		}

		break;
	}

	if (tNum != nNum) {
		printf ("Error: Could not find patch info for this file.\n");
		fail = 1;
	}
}


void db_check()
{
	int i;
	FILE *fz;
	char szLine[128];
	char tmp[32];

	fz = fopen("info.dat", "rt");
	if (fz == NULL) {
		//printf ("Error: Could not find \"info.dat\".\n");
		//fail = 1;
		//return;
		write_info_dat();

		fz = fopen("info.dat", "rt");
	}

	unsigned long int tCrc = 0;

	while (1)
	{
		memset (szLine, 0, 128);

		if (fgets(szLine, 128, fz) == NULL)
			break;

		int szLen = strlen(szLine);

		if (strncmp ("[VERSION", szLine, 8) == 0)
		{
			unsigned int datver = 0;
			if (strlen(szLine) > 9) {
				memcpy (tmp, szLine + 9, 6);
				tmp[6] = '\0';
				sscanf (tmp, "%lx", &datver);
			}
			if (datver < currentver)
			{
				fclose (fz);
				remove ("info.dat");
				printf ("Error: info.dat is out of date.  Please re-open this application\n");
				printf ("so that a newer, up-to-date one can be written\n");
				fail = 1;
				return;
			}
			continue;
		}

		if (strncmp ("//", szLine, 2) == 0 || szLen < 18) // not an info line
			continue;

		// Find commas ('cause I like to do it this way)
		int c0[5], c1 = 0;
		for (i = 0; i < szLen; i++) {
			if (szLine[i] == ',') {
				c0[c1] = i;
				c1++;
			}
			if (c1 == 4 || szLine[(i+1) & 0xff] == '/')
				break;
		}

		memcpy (tmp, szLine, c0[0]);
		tmp[c0[0]] = '\0';
		sscanf (tmp, "%lx", &tCrc);

		memcpy (tmp, szLine + c0[0] + 1, c0[1]-c0[0]);
		tmp[4] = '\0';
		sscanf (tmp, "%hX", &x_flags);

		memcpy (tmp, szLine + c0[1] + 1, c0[2]-c0[1]);
		tmp[4] = '\0';
		sscanf (tmp, "%hX", &version);

		//unsigned int tp = *((unsigned int*)(tmp + 0));
		//version = ((tp >> 24) & 0x000f) | ((tp >> 12) & 0x00f0) | (tp & 0x0f00) | ((tp << 12) & 0xf000);

		memcpy (tmp, szLine + c0[2] + 1, c0[3]-c0[2]);
		tmp[(c0[3]-c0[2])-1] = '\0';
		sscanf (tmp, "%x", &pers);

		if (tCrc == nCrc || strncmp ("[PATCH]", szLine, 7) == 0)
			break;
	}

	if (tCrc != nCrc) {
		printf ("This file was not found in the database. Detect info/settings? (Yes or No)\n");
		gets (tmp);

		if ((tmp[0] & 0xdf) == 0x59)
		{
			tCrc = 0;
			x_flags = 0;
			version = 0;
			pers = 0;
			search_info();
		} else {
			fail = 1;
			fclose (fz);
			return;
		}			
	}

	print_version();

	get_patch_addresses(fz, pers);
	if (fail) {
		fclose (fz);
		return;
	}

	get_user_input();

	if (flags == 0 && ~x_flags & 0x8000) {
		printf ("Error: No changes are necessary.\n");
		fail = 1;	
	}

	fclose (fz);
}
